#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <util/atomic.h>
#include "lcd.h"

volatile int32_t overflows = 0;
volatile uint8_t pulseNumber = 1;
volatile int32_t firstPulseTime = 0;
volatile int32_t secondPulseTime = 0;
volatile int32_t period = 0;

int main(void) {
	lcdInit();
	lcdInitPrintf();
	//wcz przerwania globalnie
	sei();
	//CLK/1
	TCCR1B = _BV(CS10);
	//przerwania na przepenienie i input capture
	TIMSK = _BV(TOIE1) | _BV(TICIE1); 
	while(1) {
		float f = 0;
		//obliczenie czstotliwoci
		if (period) f = 16000000.0 / period;
		lcdGotoXY(0, 0);
		printf("%9.2f Hz", f);
		_delay_ms(500);
	}
}

ISR(TIMER1_OVF_vect){
	overflows++;
}

ISR(TIMER1_CAPT_vect){
	if (pulseNumber == 1) {
		firstPulseTime = (uint16_t)ICR1;
		pulseNumber = 2;
		overflows = 0;
	} else {
		 secondPulseTime = (uint16_t)ICR1;
		 pulseNumber = 1;
		 period = secondPulseTime - firstPulseTime + overflows * 65536;
	}
}